package org.netkernel.json.core.endpoint;

import java.io.ByteArrayOutputStream;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.netkernel.json.core.util.HDS2Utils;
import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class JSONFromHDS extends StandardAccessorImpl
{
	public static final String ARRAY_INDICATOR_SUFFIX="__A";
	
	public JSONFromHDS()
	{	declareThreadSafe();
		declareArgument(new SourcedArgumentMetaImpl("operand","input HDS document",null,new Class[]{IHDSNode.class}));
		declareSourceRepresentation(JSONObject.class);
	}

	public void onSource(INKFRequestContext aContext) throws Exception
	{
		Object rawOperand=aContext.source("arg:operand");
		Class required=aContext.getThisRequest().getRepresentationClass();
		if (rawOperand instanceof IHDSDocument && required.isAssignableFrom(ByteArrayRepresentation.class))
		{	ByteArrayOutputStream baos=new ByteArrayOutputStream(1024);
			HDS2Utils.serializeHDS(baos, (IHDSDocument)rawOperand);
			aContext.createResponseFrom(new ByteArrayRepresentation(baos));
		}
		else
		{	if (! (rawOperand instanceof IHDSNode))
			{	rawOperand=aContext.transrept(rawOperand, IHDSNode.class);
			}
			IHDSNode hds=(IHDSNode)rawOperand;
			JSONObject result;
			Object o=recurseHDSToJSON(hds);
			if (o instanceof JSONObject)
			{	result=(JSONObject)o;
			}
			else
			{	result = new JSONObject();
				result.put(hds.getName(), o);
			}
			aContext.createResponseFrom(result);
		}
	}
	
	private static Object recurseHDSToJSON(IHDSNode  aHDS) throws JSONException
	{
		Object result=null;
		
		String name=aHDS.getName();
		Object value=aHDS.getValue();
		if (value==null)
		{	if (name!=null && name.endsWith(ARRAY_INDICATOR_SUFFIX))
			{	JSONArray a = new JSONArray();
				for (IHDSNode n : aHDS.getChildren())
				{	Object c=recurseHDSToJSON(n);
					if (c!=null) a.put(c);
				}
				result=a;
			}
			
			else if (aHDS.getChildren().length>0)
			{	JSONObject o = new JSONObject();
				for (IHDSNode n : aHDS.getChildren())
				{	Object c=recurseHDSToJSON(n);
					if (c!=null)
					{	String cname=n.getName();
						if (cname.endsWith(ARRAY_INDICATOR_SUFFIX))
						{	cname=cname.substring(0,cname.length()-ARRAY_INDICATOR_SUFFIX.length());
						}
						o.put(cname, c);
					}
				}
				result=o;
			}
			else
			{	if (aHDS.getParent()==null)
				{	result=new JSONObject();
				}
				else
				{	result=JSONObject.NULL;
				}
			}
		}
		else if (value instanceof String || value instanceof Boolean || value instanceof Number)
		{	result=value;
		}
		return result;
	}	
}
